@@ -33,12 +33,14 @@
 #include <thread>
 #include <algorithm>
 #include <memory>
+#include <mutex>
 #include <wx/wx.h>
+#include <wx/event.h>
 #include <wx/dir.h>
 #include <vector>
+#include <wx/object.h>
 #include <wx/regex.h>
 #include <wx/filename.h>
-#include <vector>
 #include <wx/hashmap.h>
 #include "Configuration.h"
 #include "precomp.h"
@@ -52,7 +54,7 @@
    - all words that appear in the worksheet
    - and a list of maxima's builtin commands.
 */
-class AutoComplete
+class AutoComplete : public wxEvtHandler
 {
 #if wxCHECK_VERSION(3, 3, 0) || wxUSE_STL
   typedef std::unordered_map <wxString, int> WorksheetWords;
@@ -117,10 +119,12 @@ public:
   std::vector<wxString> CompleteSymbol(wxString partial, autoCompletionType type = command);
   //! Basically runs a regex over templates
   static wxString FixTemplate(wxString templ);
+  //! Returns a list of demo files we know of
+  std::vector<wxString> GetDemoFilesList();
+  //! Does a demo file for this command exist?
+  bool HasDemofile(wxString commandname);
 
 private:
-  //! An AddSymbol that doesn't wait for background tasks to finish
-  void AddSymbol_nowait(wxString fun, autoCompletionType type = command);
   //! The configuration storage
   Configuration *m_configuration;
   //! Loads the list of loadable files and can be run in a background task
@@ -139,15 +143,20 @@ private:
   class GetGeneralFiles : public wxDirTraverser
   {
   public:
-    explicit GetGeneralFiles(std::vector<wxString>& files, wxString prefix = wxEmptyString) :
-      m_files(files), m_prefix(prefix) { }
+    explicit GetGeneralFiles(std::vector<wxString>& files,
+                             std::mutex *lock,
+                             wxString prefix = wxEmptyString) :
+      m_files(files), m_lock(lock), m_prefix(prefix) { }
     wxDirTraverseResult OnFile(const wxString& filename) override
       {
         wxFileName newItemName(filename);
         wxString newItem = "\"" + m_prefix + newItemName.GetFullName() + "\"";
         newItem.Replace(wxFileName::GetPathSeparator(), "/");
-        if(std::find(m_files.begin(), m_files.end(), newItem) == m_files.end())
-          m_files.push_back(newItem);
+        {
+          const std::lock_guard<std::mutex> lock(*m_lock);
+          if(std::find(m_files.begin(), m_files.end(), newItem) == m_files.end())
+            m_files.push_back(newItem);
+        }
         return wxDIR_CONTINUE;
       }
     wxDirTraverseResult OnDir(const wxString& dirname) override
@@ -155,13 +164,20 @@ private:
         wxFileName newItemName(dirname);
         wxString newItem = "\"" + m_prefix + newItemName.GetFullName() + "/\"";
         newItem.Replace(wxFileName::GetPathSeparator(), "/");
-        if(std::find(m_files.begin(), m_files.end(), newItem) == m_files.end())
-          m_files.push_back(newItem);
+        {
+          const std::lock_guard<std::mutex> lock(*m_lock);
+          if(std::find(m_files.begin(), m_files.end(), newItem) == m_files.end())
+            m_files.push_back(newItem);
+        }
         return wxDIR_IGNORE;
       }
-    std::vector<wxString>& GetResult(){return m_files;}
+    std::vector<wxString> GetResult(){
+      const std::lock_guard<std::mutex> lock(*m_lock);
+      return m_files;
+    }
   protected:
     std::vector<wxString>& m_files;
+    std::mutex *m_lock;
     wxString m_prefix;
   };
 
@@ -169,8 +185,10 @@ private:
   class GetMacFiles_includingSubdirs : public wxDirTraverser
   {
   public:
-    explicit GetMacFiles_includingSubdirs(std::vector<wxString>& files, wxString prefix = wxEmptyString) :
-      m_files(files), m_prefix(prefix)  { }
+    explicit GetMacFiles_includingSubdirs(std::vector<wxString>& files,
+                                          std::mutex *lock,
+                                          wxString prefix = wxEmptyString) :
+      m_files(files), m_lock(lock), m_prefix(prefix)  { }
     wxDirTraverseResult OnFile(const wxString& filename) override
       {
         if(
@@ -182,8 +200,11 @@ private:
           wxFileName newItemName(filename);
           wxString newItem = "\"" + m_prefix + newItemName.GetName() + "\"";
           newItem.Replace(wxFileName::GetPathSeparator(), "/");
-          if(std::find(m_files.begin(), m_files.end(), newItem) == m_files.end())
-            m_files.push_back(newItem);
+          {
+            const std::lock_guard<std::mutex> lock(*m_lock);
+            if(std::find(m_files.begin(), m_files.end(), newItem) == m_files.end())
+              m_files.push_back(newItem);
+          }
         }
         return wxDIR_CONTINUE;
       }
@@ -199,9 +220,13 @@ private:
         else
           return wxDIR_CONTINUE;
       }
-    std::vector<wxString>& GetResult(){return m_files;}
+    std::vector<wxString> GetResult(){
+      const std::lock_guard<std::mutex> lock(*m_lock);
+      return m_files;
+    }
   protected:
     std::vector<wxString>& m_files;
+    std::mutex *m_lock;
     wxString m_prefix;
   };
 
@@ -209,15 +234,20 @@ private:
   class GetMacFiles : public GetMacFiles_includingSubdirs
   {
   public:
-    explicit GetMacFiles(std::vector<wxString>& files, wxString prefix = wxEmptyString) :
-      GetMacFiles_includingSubdirs(files, prefix){ }
+    explicit GetMacFiles(std::vector<wxString>& files,
+                         std::mutex *lock,
+                         wxString prefix = wxEmptyString) :
+      GetMacFiles_includingSubdirs(files, lock, prefix){ }
     wxDirTraverseResult OnDir(const wxString& dirname) override
       {
         wxFileName newItemName(dirname);
         wxString newItem = "\"" + m_prefix + newItemName.GetFullName() + "/\"";
         newItem.Replace(wxFileName::GetPathSeparator(), "/");
-        if(std::find(m_files.begin(), m_files.end(), newItem) == m_files.end())
-          m_files.push_back(newItem);
+        {
+          const std::lock_guard<std::mutex> lock(*m_lock);
+          if(std::find(m_files.begin(), m_files.end(), newItem) == m_files.end())
+            m_files.push_back(newItem);
+        }
         return wxDIR_IGNORE;
       }
   };
@@ -226,8 +256,10 @@ private:
   class GetDemoFiles_includingSubdirs : public wxDirTraverser
   {
   public:
-    explicit GetDemoFiles_includingSubdirs(std::vector<wxString>& files, wxString prefix = wxEmptyString) :
-      m_files(files), m_prefix(prefix) { }
+    explicit GetDemoFiles_includingSubdirs(std::vector<wxString>& files,
+                                           std::mutex *lock,
+                                           wxString prefix = wxEmptyString) :
+      m_files(files), m_lock(lock), m_prefix(prefix) { }
     wxDirTraverseResult OnFile(const wxString& filename) override
       {
         if(filename.EndsWith(".dem"))
@@ -235,8 +267,11 @@ private:
           wxFileName newItemName(filename);
           wxString newItem = "\"" + m_prefix + newItemName.GetName() + "\"";
           newItem.Replace(wxFileName::GetPathSeparator(), "/");
-          if(std::find(m_files.begin(), m_files.end(), newItem) == m_files.end())
-            m_files.push_back(newItem);
+          {
+            const std::lock_guard<std::mutex> lock(*m_lock);
+            if(std::find(m_files.begin(), m_files.end(), newItem) == m_files.end())
+              m_files.push_back(newItem);
+          }
         }
         return wxDIR_CONTINUE;
       }
@@ -252,9 +287,13 @@ private:
         else
           return wxDIR_CONTINUE;
       }
-    std::vector<wxString>& GetResult(){return m_files;}
+    std::vector<wxString> GetResult(){
+      const std::lock_guard<std::mutex> lock(*m_lock);
+      return m_files;
+    }
   protected:
     std::vector<wxString>& m_files;
+    std::mutex *m_lock;
     wxString m_prefix;
   };
 
@@ -262,29 +301,34 @@ private:
   class GetDemoFiles : public GetDemoFiles_includingSubdirs
   {
   public:
-    explicit GetDemoFiles(std::vector<wxString>& files, wxString prefix = wxEmptyString) :
-      GetDemoFiles_includingSubdirs(files, prefix){ }
+    explicit GetDemoFiles(std::vector<wxString>& files,
+                          std::mutex *lock,
+                          wxString prefix = wxEmptyString) :
+      GetDemoFiles_includingSubdirs(files, lock, prefix){ }
     virtual wxDirTraverseResult OnDir(const wxString& dirname) override
       {
         wxFileName newItemName(dirname);
         wxString newItem = "\"" + m_prefix + newItemName.GetFullName() + "/\"";
         newItem.Replace(wxFileName::GetPathSeparator(), "/");
-        if(std::find(m_files.begin(), m_files.end(), newItem) == m_files.end())
-          m_files.push_back(newItem);
+        {
+          const std::lock_guard<std::mutex> lock(*m_lock);
+          if(std::find(m_files.begin(), m_files.end(), newItem) == m_files.end())
+            m_files.push_back(newItem);
+        }
         return wxDIR_IGNORE;
       }
   };
-
-  void WaitForBackgroundThread_Symbols();
-  void WaitForBackgroundThread_Files();
-  void WaitForBackgroundThreads();
-
+ 
+  std::thread m_addSymbols_backgroundThread;
+  std::thread m_addFiles_backgroundThread;
+  //! Is locked when someone accesses a keyword list
+  std::mutex m_keywordsLock;
   //! The lists of autocompletable symbols for the classes defined in autoCompletionType
   std::vector<std::vector<wxString>> m_wordList;
   static wxRegEx m_args;
   WorksheetWords m_worksheetWords;
-  std::unique_ptr<std::thread> m_addSymbols_backgroundThread;
-  std::unique_ptr<std::thread> m_addFiles_backgroundThread;
 };
 
+wxDECLARE_EVENT(NEW_DEMO_FILES_EVENT, wxCommandEvent);
+
 #endif // AUTOCOMPLETE_H
