@@ -28,7 +28,7 @@
 
 #include "Configuration.h"
 
-#include "Cell.h"
+#include "cells/Cell.h"
 #include "TextStyle.h"
 #include "Dirstructure.h"
 #include "ErrorRedirector.h"
@@ -53,6 +53,7 @@
 #include <limits>
 
 Configuration::Configuration(wxDC *dc, InitOpt options) :
+  m_initOpts(options),
   m_eng{m_rd()},
   m_dc(dc)
 {
@@ -239,6 +240,8 @@ wxSize Configuration::GetPPI() const {
 }
 
 void Configuration::ResetAllToDefaults() {
+  RecalculateForce();
+  m_display2d_Unicode = true;
   m_printMargin_Top = 10;
   m_printMargin_Bot = 10;
   m_printMargin_Left = 10;
@@ -564,7 +567,11 @@ wxString Configuration::GetAutosubscript_string() const {
   }
 }
 
-void Configuration::ShowCodeCells(bool show) { m_showCodeCells = show; }
+void Configuration::ShowCodeCells(bool show) {
+  if(m_showCodeCells != show)
+    RecalculateForce();
+  m_showCodeCells = show;
+}
 
 void Configuration::SetBackgroundBrush(wxBrush brush) {
   m_BackgroundBrush = brush;
@@ -598,6 +605,7 @@ bool Configuration::MaximaFound(wxString location) {
 }
 
 void Configuration::ReadConfig() {
+  RecalculateForce();
   wxConfigBase *config = wxConfig::Get();
 
   config->Read(wxS("configID"), &m_configId);
@@ -1067,6 +1075,8 @@ void Configuration::SetZoomFactor(double newzoom) {
   if(m_zoomFactor == newzoom)
     return;
 
+  RecalculateForce();
+
   for (auto &i: m_styles)
     i.ClearCache();
   if (newzoom > GetMaxZoomFactor())
@@ -1078,8 +1088,11 @@ void Configuration::SetZoomFactor(double newzoom) {
 }
 
 Configuration::~Configuration() {
-  WriteStyles();
-  WriteSettings();
+  if(m_initOpts != temporary)
+    {
+      WriteStyles();
+      WriteSettings();
+    }
 }
 
 bool Configuration::CharsExistInFont(const wxFont &font,
@@ -1167,6 +1180,7 @@ wxString Configuration::MaximaDefaultLocation() {
 }
 
 void Configuration::ReadStyles(const wxString &file) {
+  RecalculateForce();
   wxConfigBase *config = NULL;
   if (file == wxEmptyString)
     config = wxConfig::Get();
