@@ -37,33 +37,33 @@
 
 #include "MathParser.h"
 
-#include "AbsCell.h"
-#include "AnimationCell.h"
-#include "AtCell.h"
-#include "BoxCell.h"
-#include "NamedBoxCell.h"
-#include "CellList.h"
-#include "ConjugateCell.h"
-#include "DiffCell.h"
-#include "ExptCell.h"
-#include "FunCell.h"
-#include "ImgCell.h"
-#include "IntCell.h"
-#include "IntervalCell.h"
-#include "LabelCell.h"
-#include "LimitCell.h"
-#include "ListCell.h"
-#include "LongNumberCell.h"
-#include "MatrCell.h"
-#include "ParenCell.h"
-#include "SetCell.h"
-#include "SqrtCell.h"
+#include "cells/AbsCell.h"
+#include "cells/AnimationCell.h"
+#include "cells/AtCell.h"
+#include "cells/BoxCell.h"
+#include "cells/NamedBoxCell.h"
+#include "cells/CellList.h"
+#include "cells/ConjugateCell.h"
+#include "cells/DiffCell.h"
+#include "cells/ExptCell.h"
+#include "cells/FunCell.h"
+#include "cells/ImgCell.h"
+#include "cells/IntCell.h"
+#include "cells/IntervalCell.h"
+#include "cells/LabelCell.h"
+#include "cells/LimitCell.h"
+#include "cells/ListCell.h"
+#include "cells/LongNumberCell.h"
+#include "cells/MatrCell.h"
+#include "cells/ParenCell.h"
+#include "cells/SetCell.h"
+#include "cells/SqrtCell.h"
 #include "StringUtils.h"
-#include "SubCell.h"
-#include "SubSupCell.h"
-#include "SumCell.h"
-#include "Version.h"
-#include "VisiblyInvalidCell.h"
+#include "cells/SubCell.h"
+#include "cells/SubSupCell.h"
+#include "cells/SumCell.h"
+#include "cells/ProductCell.h"
+#include "cells/VisiblyInvalidCell.h"
 
 /*! Calls a member function from a function pointer
 
@@ -1072,8 +1072,6 @@ std::unique_ptr<Cell> MathParser::ParseSumTag(wxXmlNode *node) {
   wxXmlNode *child = node->GetChildren();
   child = SkipWhitespaceNode(child);
   wxString type = node->GetAttribute(wxS("type"), wxS("sum"));
-  sumStyle style =
-    ((type == wxS("prod")) || (type == wxS("lprod"))) ? SM_PROD : SM_SUM;
   auto highlight = m_highlight;
 
   auto under = HandleNullPointer(ParseTag(child, false));
@@ -1084,9 +1082,15 @@ std::unique_ptr<Cell> MathParser::ParseSumTag(wxXmlNode *node) {
   child = GetNextTag(child);
   auto base = HandleNullPointer(ParseTag(child, false));
 
-  auto sum = std::make_unique<SumCell>(m_group, m_configuration, style,
-                                       std::move(under), std::move(over),
-                                       std::move(base));
+  std::unique_ptr<Cell> sum;
+  if ((type == wxS("prod")) || (type == wxS("lprod")))
+    sum = std::make_unique<ProductCell>(m_group, m_configuration, 
+                                    std::move(under), std::move(over),
+                                    std::move(base));
+  else
+    sum = std::make_unique<SumCell>(m_group, m_configuration, 
+                                    std::move(under), std::move(over),
+                                    std::move(base));
   sum->SetHighlight(highlight);
   sum->SetType(m_ParserStyle);
   sum->SetStyle(TS_VARIABLE);
@@ -1143,16 +1147,15 @@ std::unique_ptr<Cell> MathParser::ParseTableTag(wxXmlNode *node) {
     matrix->ColNames(true);
   if (node->GetAttribute(wxS("rownames"), wxS("false")) == wxS("true"))
     matrix->RowNames(true);
-  if (node->GetAttribute(wxS("roundedParens")) == wxS("false"))
-    matrix->BracketParens();
-  if (node->GetAttribute(wxS("roundedParens")) == wxS("true"))
-    matrix->RoundedParens();
   if (node->GetAttribute(wxS("bracketParens")) == wxS("true"))
     matrix->BracketParens();
-  if (node->GetAttribute(wxS("angledParens")) == wxS("true"))
+  else if (node->GetAttribute(wxS("angledParens")) == wxS("true"))
     matrix->AngledParens();
-  if (node->GetAttribute(wxS("straightParens")) == wxS("true"))
+  else if (node->GetAttribute(wxS("straightParens")) == wxS("true"))
     matrix->StraightParens();
+  else if (node->GetAttribute(wxS("roundedParens"), wxS("true")) == wxS("true"))
+    matrix->RoundedParens();
+  else matrix->NoParens();
 
   wxXmlNode *rows = SkipWhitespaceNode(node->GetChildren());
   while (rows) {
