@@ -72,7 +72,7 @@ Maxima::Maxima(wxSocketBase *socket) :
 
 Maxima::~Maxima() { m_socket->Close(); }
 
-bool Maxima::Write(const void *buffer, size_t length) {
+bool Maxima::Write(const void *buffer, std::size_t length) {
   if (!m_socketOutputData.IsEmpty()) {
     if (buffer && length)
       m_socketOutputData.AppendData(buffer, length);
@@ -111,7 +111,7 @@ void Maxima::SocketEvent(wxSocketEvent &event) {
   case wxSOCKET_OUTPUT:
     if (Write(nullptr, 0)) {
       MaximaEvent *discEvent =
-	new MaximaEvent(MaximaEvent::WRITE_PENDING, this);
+        new MaximaEvent(MaximaEvent::WRITE_PENDING, this);
       QueueEvent(discEvent);
     }
     break;
@@ -168,7 +168,7 @@ void Maxima::ReadSocket() {
       std::cerr << m_socketInputData;
       std::cerr.flush();
     }
-  
+
   if (m_first || wxm::EndsWithChar(m_socketInputData, '\n') ||
       m_socketInputData.EndsWith(wxMaxima::m_promptSuffix)) {
     m_stringEndTimer.Stop();
