@@ -22,7 +22,7 @@
 
 /*! \file
 
-  This file contains the definition of the class History that handles the recently 
+  This file contains the definition of the class History that handles the recently
   issued commands for the history pane.
 */
 #include "precomp.h"
@@ -60,7 +60,7 @@ public:
 
   //! Update the bitmaps to the Right size for the Resolution
   void UpdateBitmaps();
-  
+
   //! Informs the status bar about networking events.
   void NetworkStatus(networkState status);
 
@@ -95,7 +95,7 @@ public:
     disconnected
   };
 
-  void UpdateStatusMaximaBusy(MaximaStatus status, long bytesFromMaxima);
+  void UpdateStatusMaximaBusy(MaximaStatus status, std::size_t bytesFromMaxima);
   void SetStatusText(wxString statusText){m_statusText->SetLabel(statusText);}
 protected:
   void StatusMsgDClick(wxCommandEvent &ev);
@@ -105,7 +105,7 @@ protected:
   void HandleTimerEvent();
 
 private:
-  std::unique_ptr<struct NSVGrasterizer, decltype(std::free)*> m_svgRast{nullptr, std::free};
+  std::unique_ptr<struct wxm_NSVGrasterizer, decltype(std::free)*> m_svgRast{nullptr, std::free};
   //! The display resolution
   wxSize m_ppi;
   /*! How many percents of the available CPU power does maxima use?
@@ -154,7 +154,7 @@ private:
   wxTimer SendTimer;
   //! The timer that prolongs the showing of the "receiving" bitmap a bit.
   wxTimer ReceiveTimer;
-  
+
   wxBitmap m_bitmap_waitForStart;
   wxBitmap m_bitmap_process_wont_start;
   wxBitmap m_bitmap_sending;
