@@ -47,18 +47,18 @@
 #include <memory>
 #include <list>
 #include "CellPointers.h"
-#include "VariablesPane.h"
+#include "sidebars/VariablesPane.h"
 #include "Notification.h"
-#include "Cell.h"
-#include "EditorCell.h"
-#include "GroupCell.h"
-#include "TextCell.h"
+#include "cells/Cell.h"
+#include "cells/EditorCell.h"
+#include "cells/GroupCell.h"
+#include "cells/TextCell.h"
 #include "EvaluationQueue.h"
 #include "FindReplaceDialog.h"
 #include "Autocomplete.h"
 #include "AutocompletePopup.h"
-#include "TableOfContents.h"
-#include "UnicodeSidebar.h"
+#include "sidebars/TableOfContents.h"
+#include "sidebars/UnicodeSidebar.h"
 #include "ToolBar.h"
 #include <thread>
 #include <vector>
@@ -487,7 +487,7 @@ private:
     as long as there is a mutex watching that we do the blit when no-one else
     accesses the DC should be fine.
   */
-  void DrawGroupCell_UsingBitmap(wxDC *dc, GroupCell *cell, wxRect DrawRegion);
+  void DrawGroupCell_UsingBitmap(wxDC *dc, GroupCell *cell);
 
   //! All that has need to be done before drawing a GroupCell in a DC
   void PrepareDrawGC(wxDC &dc);
@@ -906,9 +906,6 @@ public:
 
   void Recalculate() { Recalculate(GetTree()); }
 
-  //! Schedule a full recalculation of the worksheet
-  void RecalculateForce();
-
   /*! Empties the current document
 
     Used before opening a new file or when the "new" button is pressed.
