@@ -36,7 +36,6 @@
 #include <wx/image.h>
 #include <wx/timer.h>
 
-#include <wx/filesys.h>
 #include <wx/fs_arc.h>
 #include <wx/mstream.h>
 #include <wx/wfstream.h>
@@ -48,17 +47,17 @@ class AnimationCell final : public ImgCellBase
 public:
   /*! The constructor
 
-    \param framerate The individual frame rate that has to be set for this cell only. 
-    If the default frame rate from the config is to be used instead this parameter 
+    \param framerate The individual frame rate that has to be set for this cell only.
+    If the default frame rate from the config is to be used instead this parameter
     has to be set to -1.
-    \param config A pointer to the pointer to the configuration storage of the 
+    \param config A pointer to the pointer to the configuration storage of the
     worksheet this cell belongs to.
-    \param filesystem The filesystem the contents of this animation can be found in.
-    NULL = the operating system's filesystem
+    \param wxmxFile The wxmx file the contents of this animation can be found in.
+    wxEmptyString = no wxmx file, but the system's filesystem instead
     \param group     The parent GroupCell this cell belongs to.
   */
   AnimationCell(GroupCell *group, Configuration *config,
-                std::shared_ptr<wxFileSystem> &filesystem, int framerate = -1);
+                const wxString &wxmxFile, int framerate = -1);
   AnimationCell(GroupCell *group, Configuration *config, int framerate = -1);
   AnimationCell(GroupCell *group, const AnimationCell &cell);
   //! A constructor that loads the compressed file from a wxMemoryBuffer
@@ -69,13 +68,13 @@ public:
   AnimationCell(const AnimationCell&) = delete;
 
   void SetConfiguration(Configuration *config) override;
-  int GetPPI() const override {if(IsOk()) return m_images[m_displayed]->GetPPI(); else return 0;}
+  int GetPPI() const override {if(IsOk()) return m_images.at(m_displayed)->GetPPI(); else return 0;}
   size_t GetOriginalWidth() const override {
-    if(IsOk())return m_images[m_displayed]->GetOriginalWidth(); else return 0;}
+    if(IsOk())return m_images.at(m_displayed)->GetOriginalWidth(); else return 0;}
   size_t GetOriginalHeight() const override {
-    if(IsOk())return m_images[m_displayed]->GetOriginalHeight(); else return 0;}
+    if(IsOk())return m_images.at(m_displayed)->GetOriginalHeight(); else return 0;}
   wxString GetExtension() const override
-    { if (IsOk())return m_images[m_displayed]->GetExtension(); else return wxEmptyString; }
+    { if (IsOk())return m_images.at(m_displayed)->GetExtension(); else return wxEmptyString; }
 
   const CellTypeInfo &GetInfo() override;
   ~AnimationCell();
@@ -88,7 +87,7 @@ public:
   static wxDataFormat m_gifFormat;
 
   //! Can the current image be exported in SVG format?
-  bool CanExportSVG() const override {return (m_images[m_displayed] != NULL) && m_images[m_displayed]->CanExportSVG();}
+  bool CanExportSVG() const override {return (m_images.at(m_displayed) != NULL) && m_images.at(m_displayed)->CanExportSVG();}
 
   //! A Gif object for the clipboard
   class GifDataObject : public wxCustomDataObject
@@ -103,12 +102,12 @@ public:
   };
 
   bool IsOk() const;
-  
+
   const wxString &GetToolTip(wxPoint point) const override;
 
   /*! Remove all cached scaled images from memory
 
-    To be called when the animation is outside of the displayed portion 
+    To be called when the animation is outside of the displayed portion
     of the screen; The bitmaps will be re-generated when needed.
   */
   void ClearCache() override;
@@ -129,13 +128,13 @@ public:
   wxSize ToGif(wxString file);
 
   bool CopyToClipboard() const override;
-  
+
   //! Put the animation on the clipboard.
   bool CopyAnimationToClipboard();
 
   /*! Get the frame rate of this AnimationCell [in Hz].
 
-    Returns either the frame rate set for this slide show cell individually or 
+    Returns either the frame rate set for this slide show cell individually or
     the default frame rate chosen in the config.
   */
   int GetFrameRate() const;
@@ -147,13 +146,13 @@ public:
   void ReloadTimer();
 
   /*! Stops the timer
-    
+
     Also deletes the timer as on MSW there aren't many timers available.
   */
   void StopTimer();
 
   /*! Set the frame rate of this AnimationCell [in Hz].
-    
+
     \param Freq The requested frequency [in Hz] or -1 for: Use the default value.
     \return The frame rate that was actually set.
   */
@@ -162,31 +161,30 @@ public:
   bool AnimationRunning() const { return m_animationRunning; }
   void AnimationRunning(bool run);
   bool CanPopOut() const override
-    { return (!m_images[m_displayed]->GnuplotSource().empty()); }
+    { return m_images.at(m_displayed) && (m_images.at(m_displayed)->HasGnuplotSource()); }
 
   void GnuplotSource(int image, wxString gnuplotFilename, wxString dataFilename,
-                     std::shared_ptr<wxFileSystem> &filesystem)
+                     const wxString &wxmxFile)
     { m_images[image]->GnuplotSource(std::move(gnuplotFilename),
-                                     std::move(dataFilename), filesystem); }
+                                     std::move(dataFilename), wxmxFile); }
   void CompressedGnuplotSource(int image, wxString gnuplotFilename, wxString dataFilename,
-                               std::shared_ptr<wxFileSystem> &filesystem)
+                               const wxString &wxmxFile)
     { m_images[image]->CompressedGnuplotSource(std::move(gnuplotFilename),
                                                std::move(dataFilename),
-                                               filesystem); }
+                                               wxmxFile); }
 
   wxString GnuplotSource() const override
     {
-      if (!m_images[m_displayed])
+      if (!m_images.at(m_displayed))
         return wxEmptyString;
       else
-        return m_images[m_displayed]->GnuplotSource();
+        return m_images.at(m_displayed)->GnuplotSource();
     }
 
 private:
   CellPointers *const m_cellPointers = GetCellPointers(); // must come before m_timer (!)
   wxTimer m_timer;
   std::vector<std::shared_ptr<Image>> m_images;
-  std::shared_ptr<wxFileSystem> m_fileSystem;
 
   /*! The framerate of this cell.
 
@@ -198,16 +196,16 @@ private:
 
 //** Bitfield objects (1 bytes)
 //**
-  void InitBitFields()
+  void InitBitFields_AnimationCell()
     { // Keep the initialization order below same as the order
       // of bit fields in this class!
       m_animationRunning = true;
       m_drawBoundingBox = false;
     }
 
-  bool m_animationRunning : 1 /* InitBitFields */;
-  bool m_drawBoundingBox : 1 /* InitBitFields */;
-
+  bool m_animationRunning : 1 /* InitBitFields_AnimationCell */;
+  bool m_drawBoundingBox : 1 /* InitBitFields_AnimationCell */;
+  wxString m_wxmxFile;
 
   int GetImageBorderWidth() const override { return m_imageBorderWidth; }
 
@@ -221,11 +219,11 @@ private:
   wxString ToTeX() const override;
   wxString ToXML() const override;
 
-  double GetMaxWidth() const override;
-  double GetHeightList() const override;
-  void SetMaxWidth(double width) override;
-  void SetMaxHeight(double height) override;
-  
+  wxCoord GetMaxWidth() const override;
+  wxCoord GetHeightList() const override;
+  void SetMaxWidth(wxCoord width) override;
+  void SetMaxHeight(wxCoord height) override;
+
   void DrawBoundingBox(wxDC &WXUNUSED(dc), bool WXUNUSED(all) = false)  override
     { m_drawBoundingBox = true; }
 };
