@@ -57,7 +57,7 @@ const wxString &Cell::GetToolTip(const wxPoint point) const {
         return toolTip;
     }
 
-  return *m_toolTip;
+  return GetLocalToolTip();
 }
 
 Cell::Cell(GroupCell *group, Configuration *config)
@@ -240,71 +240,56 @@ GroupCell *Cell::GetGroup() const {
 bool Cell::NeedsRecalculation(AFontSize fontSize) const {
   if (!HasValidSize())
     return true;
-  if (GetType() == MC_TYPE_GROUP)
-    return false;
-  if(m_configuration->GetVisibleRegion().GetWidth() != m_visibleRegionWidth_last)
-    return false;
-  return !EqualToWithin(Scale_Px(fontSize), m_fontSize_Scaled, 0.1f);
+  if(!EqualToWithin(Scale_Px(fontSize), m_fontSize_Scaled, 0.1f))
+    return true;
+  return(ConfigChanged());
 }
 
 int Cell::GetCenterList() const {
-  if (m_maxCenter.IsInvalid()) {
-    int maxCenter = 0;
-    for (const Cell &tmp : OnDrawList(this)) {
-      if ((&tmp != this) && (tmp.m_breakLine))
-        break;
-      if (!tmp.m_isBrokenIntoLines)
-        maxCenter = std::max(maxCenter, tmp.m_center);
-    }
-    m_maxCenter.SetCached(maxCenter);
+  int maxCenter = 0;
+  for (const Cell &tmp : OnDrawList(this)) {
+    if ((&tmp != this) && (tmp.m_breakLine))
+      break;
+    if (!tmp.m_isBrokenIntoLines)
+      maxCenter = std::max(maxCenter, tmp.m_center);
   }
-  return m_maxCenter;
+  return maxCenter;
 }
 
 int Cell::GetMaxDrop() const {
-  if (m_maxDrop.IsInvalid()) {
-    int maxDrop = 0;
-    for (const Cell &tmp : OnDrawList(this)) {
-      if ((&tmp != this) && (tmp.m_breakLine))
-        break;
-      if (!tmp.m_isBrokenIntoLines)
-        maxDrop = std::max(maxDrop, tmp.m_height - tmp.m_center);
-    }
-    m_maxDrop.SetCached(maxDrop);
+  int maxDrop = 0;
+  for (const Cell &tmp : OnDrawList(this)) {
+    if ((&tmp != this) && (tmp.m_breakLine))
+      break;
+    if (!tmp.m_isBrokenIntoLines)
+      maxDrop = std::max(maxDrop, tmp.m_height - tmp.m_center);
   }
-  return m_maxDrop;
+  return maxDrop;
 }
 
 int Cell::GetHeightList() const { return GetCenterList() + GetMaxDrop(); }
 
 int Cell::GetFullWidth() const {
-  if (m_fullWidth.IsInvalid()) {
-    // We begin this calculation with a negative offset since the full width of
-    // only a single cell doesn't contain the space that separates two cells -
-    // that is automatically added to every cell in the next step.
-    int fullWidth = 0;
-    for (const Cell &tmp : OnDrawList(this)) {
-      fullWidth += tmp.m_width;
-    }
-    m_fullWidth.SetCached(fullWidth);
+  // We begin this calculation with a negative offset since the full width of
+  // only a single cell doesn't contain the space that separates two cells -
+  // that is automatically added to every cell in the next step.
+  int fullWidth = 0;
+  for (const Cell &tmp : OnDrawList(this)) {
+    fullWidth += tmp.m_width;
   }
-  return m_fullWidth;
+  return fullWidth;
 }
 
 int Cell::GetLineWidth() const {
-  if (m_lineWidth.IsInvalid()) {
-    int width = m_width;
-    for (const Cell &tmp : OnDrawList(this)) {
-      if (&tmp != this)
-        if (tmp.m_isBrokenIntoLines || tmp.m_breakLine ||
-            (tmp.m_type == MC_TYPE_MAIN_PROMPT))
-          break;
-
-      width += tmp.m_width;
-    }
-    m_lineWidth.SetCached(width);
+  int width = m_width;
+  for (const Cell &tmp : OnDrawList(this)) {
+    if (&tmp != this)
+      if (tmp.m_isBrokenIntoLines || tmp.m_breakLine ||
+          (tmp.m_type == MC_TYPE_MAIN_PROMPT))
+        break; 
+    width += tmp.m_width;
   }
-  return m_lineWidth;
+  return width;
 }
 
 /*! Draw this cell to dc
@@ -312,12 +297,12 @@ int Cell::GetLineWidth() const {
   To make this work each derived class must draw the content of the cell
   and then call MathCall::Draw(...).
 */
-void Cell::Draw(wxPoint point, wxDC *dc, wxDC *antialiassingDC) {
+void Cell::Draw(wxPoint point, wxDC *dc, wxDC *WXUNUSED(antialiassingDC)) {
   if(m_configuration->GetDebugmode())
     {
       if(!m_isHidden)
         {
-          wxASSERT(m_width >= 0);
+          wxASSERT(m_width  >= 0);
           wxASSERT(m_height >= 0);
           wxASSERT(m_center >= 0);
         }
@@ -400,6 +385,12 @@ void Cell::SetIsExponentList() {
   }
 }
 
+void Cell::SetIsExponent() {
+  for (Cell &tmp : OnInner(this)) {
+    tmp.SetIsExponentList();
+  }
+}
+
 void Cell::DrawList(wxPoint point, wxDC *dc, wxDC *adc) {
   for (Cell &tmp : OnDrawList(this)) {
     tmp.Draw(point, dc, adc);
@@ -414,15 +405,19 @@ void Cell::RecalculateList(AFontSize fontsize) {
 
 void Cell::ResetSizeList() {
   for (Cell &tmp : OnList(this))
-    for (Cell &cell : OnInner(&tmp))
-      cell.ResetSize();
+    {
+      tmp.ResetSize();
+      for (Cell &cell : OnInner(&tmp))
+        cell.ResetSize();
+    }
 }
 
 void Cell::Recalculate(AFontSize fontsize) {
-  m_fontSize_Scaled = Scale_Px(fontsize);
-  m_visibleRegionWidth_last = m_configuration->GetVisibleRegion().GetWidth();
-  ResetCellListSizes();
-  m_recalculateWidths = false;
+  if(NeedsRecalculation(fontsize))
+    {
+      m_cellCfgCnt_last = m_configuration->CellCfgCnt();
+      m_fontSize_Scaled = Scale_Px(fontsize);
+    }
 }
 
 bool Cell::DrawThisCell(wxPoint point) {
@@ -442,7 +437,7 @@ bool Cell::DrawThisCell(wxPoint point) {
 }
 
 bool Cell::HasValidSize() const {
-  return !m_recalculateWidths && m_width >= 0 && m_height >= 0 && m_center >= 0;
+  return m_width >= 0 && m_height >= 0 && m_center >= 0;
 }
 
 bool Cell::HasStaleSize() const {
@@ -464,6 +459,95 @@ void Cell::SetConfiguration(Configuration *config) {
     cell.SetConfigurationList(config);
 }
 
+int Cell::GetLineIndent() const {
+  if ((GetTextStyle() != TS_LABEL) &&
+      (GetTextStyle() != TS_USERLABEL) &&
+      (GetTextStyle() != TS_MAIN_PROMPT) &&
+      (GetTextStyle() != TS_OTHER_PROMPT) &&
+      (GetTextStyle() != TS_ASCIIMATHS) && m_configuration->IndentMaths())
+    return Scale_Px(m_configuration->GetLabelWidth()) + 2 * MC_TEXT_PADDING;
+  return 0;
+}
+
+void Cell::BreakLines_List()
+{
+  // 1st step: Tell all cells to display as beautiful 2d object, if that is
+  // possible.
+  UnBreakUpCells();
+  RecalculateList(m_configuration->GetMathFontSize());
+
+  // 2nd step: Convert all objects that are wider than a line to 1D objects that
+  // (hopefully) can be broken into lines
+  if(BreakUpCells())
+    RecalculateList(m_configuration->GetMathFontSize());
+
+  // 3rd step: Determine a sane maximum line width
+  int fullWidth = m_configuration->GetCanvasSize().x - m_configuration->GetIndent();
+  int currentWidth = GetLineIndent();
+  //  if ((this->GetTextStyle() != TS_LABEL) && (this->GetTextStyle() != TS_USERLABEL))
+  //  fullWidth -= m_configuration->GetIndent();
+
+  // We don't want the text go exactly to the right border.
+  fullWidth -= Scale_Px(5);
+
+  // Don't let the layout degenerate for small window widths
+  if (fullWidth < Scale_Px(150))
+    fullWidth = Scale_Px(150);
+
+  // 4th step: break the output into lines.
+  if (!IsHidden()) {
+    bool prevBroken = false;
+    for (Cell &tmp : OnDrawList(this)) {
+      if (prevBroken) {
+        currentWidth += tmp.GetLineIndent();
+        prevBroken = false;
+      }
+      wxCoord const cellWidth = tmp.GetWidth();
+      tmp.SoftLineBreak(false);
+      if (tmp.HasHardLineBreak() || (currentWidth + cellWidth >= fullWidth)) {
+        tmp.SoftLineBreak(true);
+        currentWidth = tmp.GetLineIndent();
+        prevBroken = true;
+      }
+      currentWidth += cellWidth;
+    }
+  }
+  ResetSize_RecursivelyList();
+}
+
+bool Cell::BreakUpCells() {  
+  int clientWidth =
+    .8 * m_configuration->GetCanvasSize().x - m_configuration->GetIndent();
+  if (clientWidth < Scale_Px(50))
+    clientWidth = Scale_Px(50);
+
+  bool lineHeightsChanged = false;
+  if (!IsHidden())
+    for (Cell &tmp : OnDrawList(this)) {
+        if (tmp.GetWidth() < 0)
+          tmp.Recalculate(m_configuration->GetMathFontSize());
+        if (tmp.GetWidth() > clientWidth)
+          lineHeightsChanged |= tmp.BreakUp();
+    }
+  return lineHeightsChanged;
+}
+
+bool Cell::UnBreakUpCells() {
+
+  bool retval = false;
+  for (Cell &tmp : OnDrawList(this)) {
+    if (tmp.IsBrokenIntoLines()) {
+      tmp.Unbreak();
+      retval = true;
+    }
+  }
+  if(retval)
+    ResetSize_RecursivelyList();
+    
+  return retval;
+}
+
+
 wxRect Cell::GetRect(bool wholeList) const {
   if (wholeList)
     return wxRect(m_currentPoint.x, m_currentPoint.y - GetCenterList(),
@@ -729,7 +813,17 @@ wxString Cell::XMLescape(wxString input) {
   input.Replace(wxS(">"), wxS("&gt;"));
   input.Replace(wxS("'"), wxS("&apos;"));
   input.Replace(wxS("\""), wxS("&quot;"));
-  return input;
+  wxString output;
+  output.reserve(input.Length());
+  for (const auto &i: input)
+    {
+      if((i < wxS('\u001F')) ||
+         ((i >= wxS('\u007F')) && (i <= wxS('\u009F'))))
+        output += wxString::Format("&#%03i;", static_cast<int>(i));
+      else
+        output += i;
+    }
+  return output;
 }
 
 wxString Cell::RTFescape(wxString input, bool MarkDown) {
@@ -935,45 +1029,20 @@ bool Cell::ContainsRect(const wxRect &sm, bool all) const {
   as well as the vertical position of the center. Then repeats this
   with
 */
-void Cell::ResetData() {
+void Cell::ResetSize_Recursively() {
   ResetSize();
   for (Cell &cell : OnInner(this))
     for (Cell &tmp : OnList(&cell))
-      tmp.ResetData();
+      tmp.ResetSize_Recursively();
 }
 
-void Cell::ResetDataList() {
+void Cell::ResetSize_RecursivelyList() {
   for (Cell &tmp : OnList(this))
-    tmp.ResetData();
-}
-
-void Cell::ResetCellListSizesList() {
-  for (const Cell &cell : OnList(this)) {
-    cell.ResetCellListSizes();
-    for (const Cell &tmp : OnInner(&cell))
-      tmp.ResetCellListSizes();
-  }
+    tmp.ResetSize_Recursively();
 }
 
 void Cell::ResetSize() {
-  m_recalculateWidths = true;
-  m_fullWidth.Invalidate();
-  m_lineWidth.Invalidate();
-  m_maxCenter.Invalidate();
-  m_maxDrop.Invalidate();
-}
-
-void Cell::ResetCellListSizes() const {
-  m_fullWidth.Invalidate();
-  m_lineWidth.Invalidate();
-  m_maxCenter.Invalidate();
-  m_maxDrop.Invalidate();
-}
-
-void Cell::SetZeroSize() {
-  m_width = 0;
-  m_height = 0;
-  m_center = 0;
+  m_cellCfgCnt_last--;
 }
 
 Cell *Cell::first() const {
@@ -998,20 +1067,21 @@ bool Cell::BreakUp() { return false; }
 
 void Cell::BreakUpAndMark() {
   wxASSERT(!m_isBrokenIntoLines);
-  Cell::BreakUp();
   if (!m_isBrokenIntoLines)
-    m_recalculateWidths = true;
-  m_isBrokenIntoLines = true;
+    {
+      Cell::BreakUp();
+      ResetSize_Recursively();
+      m_isBrokenIntoLines = true;
+    }
 }
 
 void Cell::Unbreak() {
   if (m_isBrokenIntoLines) {
-    ResetData();
-    m_recalculateWidths = true;
-    m_isBrokenIntoLines = false;
+    ResetSize_Recursively();
     // Unbreak the inner cells, too
     for (Cell &cell : OnInner(this))
       cell.UnbreakList();
+    m_isBrokenIntoLines = false;
   }
   SetNextToDraw(GetNext());
 }
