@@ -292,7 +292,8 @@ public:
 
   wxCoord GetLineWidth(size_t line, size_t pos);
 
-  //! true, if this cell's width has to be recalculated.
+  /*! true, if this cell's width has to be recalculated.
+   */
   bool IsDirty() const override
     {
       return m_isDirty;
@@ -515,10 +516,6 @@ private:
   size_t m_selectionStart = 0;
   size_t m_selectionEnd = 0;
   size_t m_lastSelectionStart = 0;
-  //! Did the zoom factor change since the last recalculation?
-  bool IsZoomFactorChanged() const;
-  //! The zoom factor we had the last time we recalculated this cell.
-  double m_lastZoomFactor = -1;
   /*! A piece of styled text for syntax highlighting
 
     A piece of styled text may be
@@ -587,8 +584,6 @@ private:
 
   void FontsChanged() override
     {
-      ResetSize();
-      ResetData();
       m_widths.clear();
     }
 
