@@ -312,7 +312,7 @@ void GroupCell::AppendInput(std::unique_ptr<Cell> &&cell) {
       CellList::AppendCell(m_inputLabel, std::move(cell));
     else if (m_inputLabel->GetNext()->GetValue().Length() == 0) {
       // AppendCell is needed due to its side effect of doing
-      // m_group->ResetData. Perhaps we can decide that SetNext alone could do
+      // m_group->ResetSize_Recursively. Perhaps we can decide that SetNext alone could do
       // something like that, as long as it wouldn't cause quadratic behavior.
       CellList::SetNext(m_inputLabel.get(), nullptr);
       wxASSERT(!m_inputLabel->GetNextToDraw());
@@ -362,7 +362,7 @@ void GroupCell::RemoveOutput() {
   UpdateCellsInGroup();
 
   m_updateConfusableCharWarnings = true;
-  ResetData();
+  ResetSize_Recursively();
 }
 
 void GroupCell::AppendOutput(std::unique_ptr<Cell> &&cell) {
@@ -456,10 +456,8 @@ bool GroupCell::Recalculate() {
       RecalculateOutput();
       m_height = m_outputRect.GetHeight() + m_inputHeight;
     }
-    ClearNeedsToRecalculateWidths();
     Cell::Recalculate(m_configuration->GetDefaultFontSize());
     m_cellsAppended = false;
-    m_clientWidth_old = m_configuration->GetCanvasSize().x;
   }
   // Move all cells that follow the current one down by the amount this cell
   // has grown.
@@ -469,9 +467,6 @@ bool GroupCell::Recalculate() {
 }
 
 void GroupCell::InputHeightChanged() {
-  ResetCellListSizes();
-  if (m_inputLabel)
-    m_inputLabel->ResetCellListSizes();
   const EditorCell *editorCell = GetEditable();
   if (editorCell == NULL)
     return;
@@ -519,7 +514,9 @@ void GroupCell::RecalculateInput() {
     m_center = 0;
     return;
   } else {
-    SetZeroSize();
+    m_width = 0;
+    m_height = 0;
+    m_center = 0;
 
     if ((m_configuration->ShowCodeCells()) || (m_groupType != GC_TYPE_CODE)) {
       if (GetEditable())
@@ -529,7 +526,6 @@ void GroupCell::RecalculateInput() {
         m_inputLabel->Recalculate(EditorFontSize());
         m_inputWidth = m_width = m_inputLabel->GetFullWidth();
         m_center = m_inputLabel->GetCenterList();
-        m_inputLabel->ResetCellListSizesList();
         m_inputHeight = m_height = m_inputLabel->GetHeightList();
       }
       m_height = m_inputHeight;
@@ -582,7 +578,6 @@ void GroupCell::RecalculateOutput() {
   // Calculate the height of the output
   for (const Cell &tmp : OnDrawList(m_output.get())) {
     if (tmp.BreakLineHere()) {
-      tmp.ResetCellListSizes();
       int height_Delta = tmp.GetHeightList();
       m_width = std::max(m_width, tmp.GetLineWidth());
       m_outputRect.width = std::max(m_outputRect.width, m_width);
@@ -599,11 +594,7 @@ void GroupCell::RecalculateOutput() {
 }
 
 bool GroupCell::NeedsRecalculation(AFontSize fontSize) const {
-  return Cell::NeedsRecalculation(fontSize) ||
-    //    (GetEditable() &&
-    //    GetEditable()->NeedsRecalculation(EditorFontSize())) ||
-    (m_clientWidth_old != m_configuration->GetCanvasSize().x) ||
-    m_cellsAppended;
+  return Cell::NeedsRecalculation(fontSize) || m_cellsAppended;
 }
 
 void GroupCell::UpdateYPositionList() {
@@ -635,7 +626,7 @@ wxPoint GroupCell::CalculateInputPosition() {
   return wxPoint(m_currentPoint.x + GetInputIndent(), m_currentPoint.y);
 }
 
-int GroupCell::GetInputIndent() {
+wxCoord GroupCell::GetInputIndent() {
   int labelWidth = 0;
   if (m_configuration->IndentMaths())
     labelWidth = Scale_Px(m_configuration->GetLabelWidth()) + MC_TEXT_PADDING;
@@ -668,7 +659,7 @@ void GroupCell::UpdateOutputPositions() {
         if (!isFirst && tmp.HasBigSkip())
           in.y += MC_LINE_SKIP;
 
-        in.x = GetCurrentPoint().x + GetLineIndent(&tmp);
+        in.x = GetCurrentPoint().x + tmp.GetLineIndent();
         in.y += drop + tmp.GetCenterList();
         drop = tmp.GetMaxDrop();
       }
@@ -681,8 +672,6 @@ void GroupCell::UpdateOutputPositions() {
 
 void GroupCell::Draw(wxPoint const point, wxDC *dc, wxDC *antialiassingDC) {
   Cell::Draw(point, dc, antialiassingDC);
-  if (NeedsRecalculation(m_configuration->GetDefaultFontSize()))
-    wxLogMessage(_("One cell wasn't recalculated before displaying it."));
   if (m_configuration->ShowBrackets())
     DrawBracket(dc, antialiassingDC);
 
@@ -731,7 +720,7 @@ void GroupCell::Draw(wxPoint const point, wxDC *dc, wxDC *antialiassingDC) {
               if (!isFirst && tmp.HasBigSkip())
                 in.y += MC_LINE_SKIP;
 
-              in.x = point.x + GetLineIndent(&tmp);
+              in.x = point.x + tmp.GetLineIndent();
               in.y += drop + tmp.GetCenterList();
               drop = tmp.GetMaxDrop();
             }
@@ -767,16 +756,6 @@ wxRect GroupCell::GetRect(bool WXUNUSED(all)) const {
                 m_height);
 }
 
-int GroupCell::GetLineIndent(const Cell *cell) const {
-  if (cell && (cell->GetTextStyle() != TS_LABEL) &&
-      (cell->GetTextStyle() != TS_USERLABEL) &&
-      (cell->GetTextStyle() != TS_MAIN_PROMPT) &&
-      (cell->GetTextStyle() != TS_OTHER_PROMPT) &&
-      (cell->GetTextStyle() != TS_ASCIIMATHS) && m_configuration->IndentMaths())
-    return Scale_Px(m_configuration->GetLabelWidth()) + 2 * MC_TEXT_PADDING;
-  return 0;
-}
-
 void GroupCell::UpdateCellsInGroup() {
   if (m_output != NULL)
     m_cellsInGroup = 2 + m_output->CellsInListRecursive();
@@ -1236,9 +1215,7 @@ wxString GroupCell::ToXML() const {
       // exception of the characters XML wants to be quoted. In reality
       // wxWidget's newline handling seems to be broken => escape newlines.
       wxString question = Cell::XMLescape(it->first);
-      question.Replace(wxS("\n"), wxS("&#10;"));
       wxString answer = Cell::XMLescape(it->second);
-      answer.Replace(wxS("\n"), wxS("&#10;"));
       str += wxString::Format(wxS(" question%li=\""), static_cast<long>(i)) +
         question + wxS("\"");
       str += wxString::Format(wxS(" answer%li=\""), static_cast<long>(i)) +
@@ -1459,56 +1436,10 @@ void GroupCell::BreakLines() {
   if (cell == NULL)
     return;
 
-  if (NeedsRecalculation(EditorFontSize()))
+  //  if (NeedsRecalculation(EditorFontSize()))
     m_output->RecalculateList(m_configuration->GetMathFontSize());
 
-  // 1st step: Tell all cells to display as beautiful 2d object, if that is
-  // possible.
-  if (UnBreakUpCells(cell)) {
-    m_output->ResetCellListSizesList();
-    m_output->RecalculateList(m_configuration->GetMathFontSize());
-  }
-
-  // 2nd step: Convert all objects that are wider than a line to 1D objects that
-  // (hopefully) can be broken into lines
-  if (BreakUpCells(cell)) {
-    m_output->ResetCellListSizesList();
-    m_output->RecalculateList(m_configuration->GetMathFontSize());
-  }
-
-  // 3rd step: Determine a sane maximum line width
-  int fullWidth = m_configuration->GetCanvasSize().x;
-  int currentWidth = GetLineIndent(cell);
-  if ((cell->GetTextStyle() != TS_LABEL) && (cell->GetTextStyle() != TS_USERLABEL))
-    fullWidth -= m_configuration->GetIndent();
-
-  // We don't want the text go exactly to the right border.
-  fullWidth -= Scale_Px(1);
-
-  // Don't let the layout degenerate for small window widths
-  if (fullWidth < Scale_Px(150))
-    fullWidth = Scale_Px(150);
-
-  // 4th step: break the output into lines.
-  if (!IsHidden()) {
-    bool prevBroken = false;
-    for (Cell &tmp : OnDrawList(cell)) {
-      if (prevBroken) {
-        currentWidth += GetLineIndent(&tmp);
-        prevBroken = false;
-      }
-      int const cellWidth = tmp.GetWidth();
-      tmp.SoftLineBreak(false);
-      if (tmp.BreakLineHere() || (currentWidth + cellWidth >= fullWidth)) {
-        tmp.SoftLineBreak(true);
-        currentWidth = 0;
-        prevBroken = true;
-      }
-      currentWidth += cellWidth;
-    }
-  }
-  m_output->ResetDataList();
-  ResetCellListSizes();
+  m_output->BreakLines_List();
 }
 
 Cell::Range GroupCell::GetCellsInOutput() const {
@@ -1529,87 +1460,6 @@ Cell::Range GroupCell::GetCellsInOutput() const {
   return r;
 }
 
-bool GroupCell::BreakUpCells(Cell *cell) const {
-  if (cell == NULL)
-    return false;
-
-  int showLength;
-  switch (m_configuration->ShowLength()) {
-  case 0:
-    showLength = 5000;
-    break;
-  case 1:
-    showLength = 10000;
-    break;
-  case 2:
-    showLength = 25000;
-    break;
-  case 3:
-    showLength = 50000;
-    break;
-  default:
-    showLength = 500;
-  }
-
-  int clientWidth =
-    .8 * m_configuration->GetCanvasSize().x - m_configuration->GetIndent();
-  if (clientWidth < Scale_Px(50))
-    clientWidth = Scale_Px(50);
-
-  // Reduce the number of steps involved in layouting big equations
-  if (m_cellsInGroup > showLength) {
-    wxLogMessage(
-                 _("Resolving to 1D layout for one cell in order to save time"));
-    return false;
-  } else {
-    bool lineHeightsChanged = false;
-    if (!IsHidden())
-      for (Cell &tmp : OnDrawList(cell)) {
-        if (tmp.GetWidth() < 0)
-          tmp.Recalculate(m_configuration->GetMathFontSize());
-        if (tmp.GetWidth() > clientWidth)
-          lineHeightsChanged |= tmp.BreakUp();
-      }
-    return lineHeightsChanged;
-  }
-}
-
-bool GroupCell::UnBreakUpCells(Cell *cell) const {
-  int showLength;
-  switch (m_configuration->ShowLength()) {
-  case 0:
-    showLength = 50;
-    break;
-  case 1:
-    showLength = 500;
-    break;
-  case 2:
-    showLength = 2500;
-    break;
-  case 3:
-    showLength = 5000;
-    break;
-  default:
-    showLength = 500;
-  }
-
-  // Reduce the number of steps involved in layouting big equations
-  if (m_cellsInGroup > showLength) {
-    wxLogMessage(_("Resolving to linear layout for one big cell in order to save time"));
-    return true;
-  }
-
-  bool retval = false;
-  for (Cell &tmp : OnDrawList(cell)) {
-    if (tmp.IsBrokenIntoLines()) {
-      tmp.Unbreak();
-      retval = true;
-    }
-  }
-
-  return retval;
-}
-
 bool GroupCell::FirstLineOnlyEditor()
 {
   return(IsHidden() && (!IsHeading()));
@@ -1978,8 +1828,10 @@ wxAccStatus GroupCell::GetLocation(wxRect &rect, int elementId) {
 void CellList::Check(const GroupCell *c) {
   if (!c)
     return;
+  //cppcheck-suppress knownPointerToBool
   wxASSERT_MSG(!c->m_next || dynamic_cast<GroupCell *>(c->m_next.get()),
                _("Bug: The successor to a GroupCell is not a GroupCell."));
+  //cppcheck-suppress knownPointerToBool
   wxASSERT_MSG(!c->m_previous || dynamic_cast<GroupCell *>(c->m_previous),
                _("Bug: The predecessor to a GroupCell is not a GroupCell."));
   CellList::Check(static_cast<const Cell *>(c));
