@@ -29,7 +29,6 @@
 #include "ImgCellBase.h"
 #include <memory>
 
-#include <wx/filesys.h>
 #include <wx/fs_arc.h>
 
 class ImgCell final : public ImgCellBase
@@ -37,7 +36,7 @@ class ImgCell final : public ImgCellBase
 public:
   ImgCell(GroupCell *group, Configuration *config);
   ImgCell(GroupCell *group, Configuration *config, const wxMemoryBuffer &image, const wxString &type);
-  ImgCell(GroupCell *group, Configuration *config, const wxString &image, std::shared_ptr<wxFileSystem> &filesystem, bool remove = true);
+  ImgCell(GroupCell *group, Configuration *config, const wxString &image, const wxString &wxmFile, bool remove = true);
 
   ImgCell(GroupCell *group, Configuration *config, const wxBitmap &bitmap);
   ImgCell(GroupCell *group, const ImgCell &cell);
@@ -49,15 +48,15 @@ public:
   ImgCell &operator=(const ImgCell&) = delete;
 
   //! Tell the image which gnuplot files it was made from
-  void GnuplotSource(wxString sourcefile, wxString datafile, std::shared_ptr<wxFileSystem> &filesystem)
+  void GnuplotSource(wxString sourcefile, wxString datafile, const wxString &wxmFile)
     { if (m_image) m_image->GnuplotSource(std::move(sourcefile),
                                           std::move(datafile),
-                                          filesystem); }
+                                          wxmFile); }
   void CompressedGnuplotSource(wxString sourcefile, wxString datafile,
-                               std::shared_ptr<wxFileSystem> &filesystem)
+                               const wxString &wxmFile)
     { if (m_image) m_image->CompressedGnuplotSource(std::move(sourcefile),
                                                     std::move(datafile),
-                                                    filesystem); }
+                                                    wxmFile); }
 
   //! The name of the file with gnuplot commands that created this file
   wxString GnuplotSource() const override
@@ -71,7 +70,7 @@ public:
   size_t GetOriginalWidth() const override {return m_image->GetOriginalWidth();}
   size_t GetOriginalHeight() const override {return m_image->GetOriginalHeight();}
 
-  void ReloadImage(const wxString &image, std::shared_ptr<wxFileSystem> &filesystem);
+  void ReloadImage(const wxString &image, const wxString &wxmFile);
 
   //! Can this image be exported in SVG format?
   bool CanExportSVG() const override {return (m_image != NULL) && m_image->CanExportSVG();}
@@ -90,13 +89,13 @@ public:
 
   /*! Removes the cached scaled image from memory
 
-    The scaled version of the image will be recreated automatically once it is 
+    The scaled version of the image will be recreated automatically once it is
     needed.
   */
   void ClearCache() override { if (m_image) m_image->ClearCache(); }
 
   const wxString &GetToolTip(wxPoint point) const override;
-  
+
   //! Sets the bitmap that is shown
   void SetBitmap(const wxBitmap &bitmap);
 
@@ -112,7 +111,7 @@ public:
   //! Returns the name of the file the image was originally created from
   wxString GetOrigImageFile() const
     { return m_origImageFile; }
-  
+
   //! Sets the name of the file the image was originally created from
   void SetOrigImageFile(wxString file)
     { m_origImageFile = file; }
@@ -120,10 +119,10 @@ public:
   //! Returns the original compressed version of the image
   wxMemoryBuffer GetCompressedImage() const { return m_image->m_compressedImage; }
 
-  double GetMaxWidth() const override { return m_image ? m_image->GetMaxWidth() : -1; }
-  double GetHeightList() const override { return m_image ? m_image->GetHeightList() : -1; }
-  void SetMaxWidth(double width) override { if (m_image) m_image->SetMaxWidth(width); }
-  void SetMaxHeight(double height) override { if (m_image) m_image->SetMaxHeight(height); }
+  wxCoord GetMaxWidth() const override { return m_image ? m_image->GetMaxWidth() : -1; }
+  wxCoord GetHeightList() const override { return m_image ? m_image->GetHeightList() : -1; }
+  void SetMaxWidth(wxCoord width) override { if (m_image) m_image->SetMaxWidth(width); }
+  void SetMaxHeight(wxCoord height) override { if (m_image) m_image->SetMaxHeight(height); }
 
   void Recalculate(AFontSize fontsize) override;
 
@@ -135,7 +134,7 @@ public:
   wxString ToTeX() const override;
   wxString ToXML() const override;
 
-  bool CanPopOut() const override { return !m_image->GnuplotSource().empty(); }
+  bool CanPopOut() const override { return m_image && m_image->HasGnuplotSource(); }
 
 private:
   void SetConfiguration(Configuration *config) override;
@@ -150,17 +149,17 @@ private:
 
 //** Bitfield objects (1 bytes)
 //**
-  void InitBitFields()
+  void InitBitFields_ImgCell()
     { // Keep the initialization order below same as the order
       // of bit fields in this class!
       m_drawRectangle = true;
       m_drawBoundingBox = false;
     }
-  bool m_drawRectangle : 1 /* InitBitFields */;
-  bool m_drawBoundingBox : 1 /* InitBitFields */;
+  bool m_drawRectangle : 1 /* InitBitFields_ImgCell */;
+  bool m_drawBoundingBox : 1 /* InitBitFields_ImgCell */;
 
   static int s_counter;
-  
+
   wxString m_origImageFile;
 };
 
