@@ -396,12 +396,6 @@ void TextCell::SetValue(const wxString &text) {
 
 AFontSize TextCell::GetScaledTextSize() const { return m_fontSize_Scaled; }
 
-bool TextCell::NeedsRecalculation(AFontSize fontSize) const {
-  return Cell::NeedsRecalculation(fontSize) ||
-    (m_keepPercent_last != m_configuration->CheckKeepPercent()) ||
-    (m_greekNamesAsLetter != m_configuration->Latin2Greek());
-}
-
 wxSize TextCell::CalculateTextSize(wxDC *const dc, const wxString &text,
                                    TextCell::TextIndex const index) {
   AFontSize const fontSize = GetScaledTextSize();
@@ -458,12 +452,9 @@ void TextCell::Recalculate(AFontSize fontsize) {
   if (NeedsRecalculation(fontsize)) {
     if (GetTextStyle() == TS_ASCIIMATHS)
       ForceBreakLine(true);
-    if ((m_keepPercent_last != m_configuration->CheckKeepPercent()) ||
-        (m_greekNamesAsLetter != m_configuration->Latin2Greek()))
+    if (ConfigChanged())
         UpdateDisplayedText();
-    m_greekNamesAsLetter = m_configuration->Latin2Greek();
-    m_keepPercent_last = m_configuration->CheckKeepPercent();
-    SetFont(m_configuration->GetRecalcDC(), m_fontSize_Scaled);
+    SetFont(m_configuration->GetRecalcDC(), Scale_Px(fontsize));
 
     wxSize sz =
       CalculateTextSize(m_configuration->GetRecalcDC(), m_displayedText, cellText);
@@ -476,14 +467,14 @@ void TextCell::Recalculate(AFontSize fontsize) {
     /// Hidden cells (multiplication * is not displayed)
     if (IsHidden() ||
         (GetHidableMultSign() && (m_configuration->HidemultiplicationSign()))) {
-      m_height = m_fontSize_Scaled.Get();
-      m_width = m_fontSize_Scaled.Get() / 4;
+      m_height = Scale_Px(fontsize).Get();
+      m_width = Scale_Px(fontsize).Get() / 4;
     }
     if (m_height < Scale_Px(4))
       m_height = Scale_Px(4);
     m_center = m_height / 2;
+    Cell::Recalculate(fontsize);
   }
-  Cell::Recalculate(fontsize);
 }
 
 void TextCell::Draw(wxPoint point, wxDC *dc, wxDC *antialiassingDC) {
