@@ -27,8 +27,9 @@
 #include "wxImagePanel.h"
 #include <wx/display.h>
 #include <wx/mstream.h>
+#include <algorithm>
 
-wxImagePanel::wxImagePanel(wxWindow *parent, unsigned char *data, size_t len)
+wxImagePanel::wxImagePanel(wxWindow *parent, unsigned char *data, std::size_t len)
   : wxPanel(parent) {
   Load(data, len);
   int ppi;
@@ -43,17 +44,17 @@ wxImagePanel::wxImagePanel(wxWindow *parent, unsigned char *data, size_t len)
 #else
   ppi = wxGetDisplayPPI().x;
 #endif
-  ppi = wxMax(ppi, 75);
+  ppi = std::max(ppi, 75);
 
   SetMinSize(
-	     wxSize(ppi * 4, m_image.GetHeight() * ppi * 4 / m_image.GetWidth()));
+             wxSize(ppi * 4, m_image.GetHeight() * ppi * 4 / m_image.GetWidth()));
 
   Connect(wxEVT_PAINT, wxPaintEventHandler(wxImagePanel::paintEvent), NULL,
           this);
   Connect(wxEVT_SIZE, wxSizeEventHandler(wxImagePanel::OnSize), NULL, this);
 }
 
-void wxImagePanel::Load(unsigned char *data, size_t len) {
+void wxImagePanel::Load(unsigned char *data, std::size_t len) {
   wxMemoryInputStream istream(data, len);
   m_image.LoadFile(istream);
   m_w = m_h = -1;
