@@ -38,7 +38,7 @@
 
 ;(format t "<wxxml-start/>")
 
-;; TODO: outchar isn't properly escaped in xml  
+;; TODO: outchar isn't properly escaped in xml
 
 (format t "<wxxml-key>~A</wxxml-key>~%" (maxima-getenv "MAXIMA_AUTH_CODE"))
 
@@ -191,10 +191,23 @@ The wxMaxima bug database is available at~%
     https://github.com/wxMaxima-developers/wxmaxima/issues~%
 Submit bug reports by following the 'New issue' link on that page."))
 
-;; Was useful when maxima's build_info() didn't contain all that info,
-;; still. Now we punt it to the maxima version of the command.
+;; wxbuild_info() provides a nicer formatted version of the build_info() command.
+;; It also includes more info, e.g. the directory information from Maxima.
 (defun $wxbuild_info ()
-  ($build_info))
+  (progv  '(seconds minute hour day month year) cl-user:*maxima-build-time*
+    (format t "~30a~a~%" "wxMaxima version:" $wxmaximaversion)
+    (format t "~30a~a~%" "using wxWidgets version:" $wxwidgetsversion)
+    (format t "~30a~a~%" "Maxima version:" *autoconf-version*)
+    (format t "~30a~4,'0d-~2,'0d-~2,'0d ~2,'0d:~2,'0d:~2,'0d~%" "Maxima build date:"
+           year month day hour minute seconds)
+    (format t "~30a~a~%" "Host type:" *autoconf-host*)
+    (format t "~30a~a ~a ~a~%" "System type:" (software-type) (software-version) (machine-type))
+    (format t "~30a~a~%" "Lisp implementation type:" (lisp-implementation-type))
+    (format t "~30a~a~%" "Lisp implementation version:" (lisp-implementation-version))
+    (format t "~30a~a~%~a~%" "wxMaxima help dir:" wxHelpDir "----------" ))
+    (format t "~%~%Maxima's idea of the directory layout is:~%")
+    (format t "~a~%" (print-directories))
+  "")
 
 ;; Tell the user where to report bugs.
 ;; TODO: Make this to maxima's bug_report() when everybody uses a maxima that shows
@@ -215,7 +228,7 @@ Submit bug reports by following the 'New issue' link on that page."))
   (format t "-------------------------------------------------------------~%")
    ; Display the 2D-formatted build information
   (let (($display2d t))
-    (displa ($build_info)))
+    (displa ($wxbuild_info)))
   (format t "-------------------------------------------------------------~%"))
 
 
@@ -270,7 +283,7 @@ Submit bug reports by following the 'New issue' link on that page."))
        (pre-superscripts-xml (if pre-superscripts (wxxml-list pre-superscripts (list "<mrow>") mrow-terminate separator-xml) (list "<none/>")))
        (post-subscripts-xml (if post-subscripts (wxxml-list post-subscripts (list "<mrow>") mrow-terminate separator-xml) (list "<none/>")))
        (post-superscripts-xml (if post-superscripts (wxxml-list post-superscripts (list "<mrow>") mrow-terminate separator-xml) (list "<none/>")))
-       (mmultiscripts-xml       
+       (mmultiscripts-xml
 	(append l (list (format nil "<mmultiscripts lisp=\"wxxml-array-with-display-properties\" altCopy=\"~A\">" (wxxml-alt-copy-text x)))
 		(wxxml base-symbol nil nil 'mparen 'mparen)
 		post-subscripts-xml post-superscripts-xml
@@ -328,8 +341,8 @@ Submit bug reports by following the 'New issue' link on that page."))
     ($put s opt '$wxxml_subscripted))
   opt)
 
-;; Returns either nil (no autosubscript needed) or the result#
-;; of the autosubscript process 
+;; Returns either nil (no autosubscript needed) or the result
+;; of the autosubscript process
 (defun subscriptp (x)
   (unless (symbolp x)
     (return-from subscriptp x))
@@ -2112,8 +2125,8 @@ Submit bug reports by following the 'New issue' link on that page."))
 
 ;; Convert a symbol to a properly xml-escaped string.
 ;;
-;; The name "a\<b" is converted to "a\&lt;b". 
-;; If s is a list of symbols in the result between them 
+;; The name "a\<b" is converted to "a\&lt;b".
+;; If s is a list of symbols in the result between them
 ;; there are spaces
 (defun symbol-to-xml (s)
   (wxxml-fix-string (format nil "~a" (maybe-invert-string-case (symbol-name (stripdollar s))))))
@@ -2189,7 +2202,7 @@ Submit bug reports by following the 'New issue' link on that page."))
   (format t "<variable>~%<name>~a</name>" (symbol-to-xml var))
   (ignore-errors
     (let (($display2d nil))
-      (mtell "<value>~M</value>" (wxxml-fix-string(eval var)))))
+      (mtell "<value>~M</value>" (wxxml-fix-string ($sconcat (eval var))))))
   (format t "</variable>"))
 
 (defun wx-print-display2d ()
@@ -2203,7 +2216,7 @@ Submit bug reports by following the 'New issue' link on that page."))
   (format t "<variables>~%<variable>~%<name>~a</name>" (wxxml-fix-string (maybe-invert-string-case var)))
   (ignore-errors
     (let (($display2d nil))
-      (mtell "<value>~M</value>" (wxxml-fix-string(meval (intern var))))))
+      (mtell "<value>~M</value>" (wxxml-fix-string ($sconcat (meval (intern var)))))))
   (format t "</variable>~%</variables>~%"))
 
 (defun wx-print-variables ()
@@ -2240,6 +2253,11 @@ Submit bug reports by following the 'New issue' link on that page."))
 					;    (mapcar (lambda(key) (if (eq (char (format nil "~a" key) 0) #\$ ) (symbol-to-xml (make-symbol key)))) (wx-list-all-maxima-vars))
 
 
+;; WxMaxima reads these variables and sets several checkboxes in the Menu according to these
+;; variables. E.g. if the $numer variable is true, the checkbox for the Menu Numeric -> Numeric output
+;; is set checked.
+;; The state of this checkbox can either be changed using the wxMaxima menu (which would emit the
+;; command numer:true$ or numer:false$) - or by setting the variable as a Maxima command.
 (defun wx-print-gui-variables ()
   (finish-output)
   (format t "<variables>")
@@ -2257,6 +2275,7 @@ Submit bug reports by following the 'New issue' link on that page."))
   (wx-print-variable '$debugmode)
   (wx-print-variable '$engineering_format_floats)
   (wx-print-variable '$wxanimate_autoplay)
+  (wx-print-variable '$display2d_unicode)
   (wx-print-display2d)
   (wx-print-variable '*alt-display2d*)
   (format t "<variable><name>*maxima-operators*</name><value>&lt;operators&gt;")
