@@ -572,34 +572,67 @@ protected:
   /*! Reads the "add variable to watch list" tag maxima can send us
    */
   void ReadAddVariables(wxString &data);
+  //! Called if maxima tells us the value of the maxima variable gentranlang.
   void VariableActionGentranlang(const wxString &value);
+  //! Called if maxima tells us the value of the maxima variable numer.
+  void VariableActionNumer(const wxString &value);
+  //! Called if maxima tells us the value of the maxima variable display2d_unicode.
+  void VariableActionDisplay2d_Unicode(const wxString &value);
+  //! Called if maxima tells us the value of the maxima variable output_format_for_help.
   void VariableActionHtmlHelp(const wxString &value);
+  //! Called if maxima tells us the value of the maxima variable opsubst.
   void VariableActionOpSubst(const wxString &value);
+  //! Called if maxima tells us the value of the maxima variable sinnpiflag.
   void VariableActionSinnpiflag(const wxString &value);
+  //! Called if maxima tells us that the maxima variable sinnpiflag is undefined.
   void VariableActionSinnpiflagUndefined();
+  //! Called if maxima tells us the value of the maxima variable logexpand.
   void VariableActionLogexpand(const wxString &value);
+  //! Called if maxima tells us where the user files lie.
   void VariableActionUserDir(const wxString &value);
+  //! Called if maxima tells us where the temp files lie.
   void VariableActionTempDir(const wxString &value);
+  //! Called if maxima tells us the value of the maxima variable debugmode.
   void VariableActionDebugmode(const wxString &value);
+  //! Called if maxima tells us the maxima version as defined by autoconf.
   void VariableActionAutoconfVersion(const wxString &value);
+  //! Called if maxima tells us the maxima build host as defined by autoconf.
   void VariableActionAutoconfHost(const wxString &value);
+  //! Called if maxima tells us the maxima info dir.
   void VariableActionMaximaInfodir(const wxString &value);
+  //! Called if maxima tells us the maxima html dir.
   void VariableActionMaximaHtmldir(const wxString &value);
+  //! Called if maxima tells us the value of the maxima variable <code>gnuplot</code>
   void VariableActionGnuplotCommand(const wxString &value);
+  //! Called if maxima tells us the maxima share dir.
   void VariableActionMaximaSharedir(const wxString &value);
+  //! Called if maxima tells us the lisp name.
   void VariableActionLispName(const wxString &value);
+  //! Called if maxima tells us the lisp version.
   void VariableActionLispVersion(const wxString &value);
+  //! Called if maxima tells us the name of a package that was loaded
   void VariableActionWxLoadFileName(const wxString &value);
+  //! Called if maxima tells us the value of the maxima variable <code>wxsubscripts</code>
   void VariableActionWxSubscripts(const wxString &value);
+  //! Called if maxima tells us the value of the maxima variable <code>lmxchar</code>
   void VariableActionLmxChar(const wxString &value);
+  //! Called if maxima tells us the value of the maxima variable <code>display2d</code>
   void VariableActionDisplay2D(const wxString &value);
+  //! Called if maxima tells us if it currently outputs XML
   void VariableActionAltDisplay2D(const wxString &value);
+  //! Called if maxima tells us the value of the maxima variable <code>algebraic</code>
   void VariableActionAlgebraic(const wxString &value);
+  //! Called if maxima tells us the value of the maxima variable <code>showtime</code>
   void VariableActionShowtime(const wxString &value);
+  //! Called if maxima tells us the value of the maxima variable <code>domain</code>
   void VariableActionDomain(const wxString &value);
+  //! Called if maxima tells us the value of the maxima variable <code>wxanimate_autoplay</code>
   void VariableActionAutoplay(const wxString &value);
+  //! Called if maxima tells us the value of the maxima variable <code>engineering_format_floats</code>
   void VariableActionEngineeringFormat(const wxString &value);
+  //! Called if maxima tells us the value of the maxima variable <code>stringdisp</code>
   void VariableActionStringdisp(const wxString &value);
+  //! Called if maxima sends us the list of known operators
   void VariableActionOperators(const wxString &value);
   wxString m_maximaVariable_wxSubscripts;
   wxString m_maximaVariable_lmxchar;
@@ -711,9 +744,7 @@ protected:
   */
   struct ServerDeleter {
     void operator()(wxSocketServer* server) const {
-      server->Close();
-      wxLogMessage(_("Closing the socket maxima could connect to!"));
-      server->Destroy();
+      server->Destroy(); // Destroy() calls Close() automatically.
     }
   };
   /*! The server maxima connects to as client
@@ -757,6 +788,12 @@ public:
   //! Remove the said window from the list of toplevel windows
   static void DelistTopLevelWindow(wxMaxima *);
 protected:
+    //! Is called when we get a new list of demo files
+  //! Is called when we get a new list of demo files
+  void OnNewDemoFiles(wxCommandEvent &event);
+  //! Is called when a demo file menu is clicked
+  void OnDemoFileMenu(wxCommandEvent &ev);
+
   void OnSize(wxSizeEvent &event);
   void OnMove(wxMoveEvent &event);
   void OnMaximize(wxCommandEvent &event);
