@@ -42,18 +42,20 @@
 #include <wx/aui/aui.h>
 #include <wx/notifmsg.h>
 #include <wx/wrapsizer.h>
+#include "sidebars/DrawSidebar.h"
 
-#include "ScrollingGenWizPanel.h"
+#include "wizards/ScrollingGenWizPanel.h"
 #include "Worksheet.h"
+#include "sidebars/SymbolsSidebar.h"
 #include "HelpBrowser.h"
 #include "RecentDocuments.h"
 #include "Version.h"
 #include "MainMenuBar.h"
-#include "History.h"
-#include "XmlInspector.h"
+#include "sidebars/History.h"
+#include "sidebars/XmlInspector.h"
 #include "StatusBar.h"
-#include "LogPane.h"
-#include "ButtonWrapSizer.h"
+#include "sidebars/LogPane.h"
+#include "sidebars/ButtonWrapSizer.h"
 #include <list>
 
 
@@ -162,6 +164,10 @@ protected:
   StatusBar *m_statusBar;
   //! The menu bar
   MainMenuBar *m_MenuBar;
+  //! The "demo" sub-menu
+  wxMenu *m_demo_sub;
+  //! The submenus for the various "demo" sub-submenus
+  std::vector<wxMenu *> m_demoSubs;
   //! The "view" menu
   wxMenu *m_viewMenu;
   //! The gentran menu
@@ -255,46 +261,6 @@ private:
 */
   void SetupMenu();
 
-  wxWindow *CreateStatPane();
-
-  wxWindow *CreateMathPane();
-
-  wxWindow *CreateFormatPane();
-
-  //! The class for the sidebar with the draw commands
-  class DrawPane: public wxScrolled<wxPanel>
-  {
-  public:
-    explicit DrawPane(wxWindow *parent, int id = wxID_ANY);
-    /*! Tell the sidebar if we currently are inside a 2D or a 3D plot command
-
-      \param dimensions
-      - 0 = We aren't inside a plot
-      - 2 = We are inside a 2D plot
-      - 3 = We are inside a 3D plot
-    */
-    void SetDimensions(int dimensions);
-    int  GetDimensions() { return m_dimensions; }
-  protected:
-    void OnSize(wxSizeEvent &event);
-  private:
-    Buttonwrapsizer *m_grid;
-    wxButton *m_draw_setup2d;
-    wxButton *m_draw_setup3d;
-    wxButton *m_draw_explicit;
-    wxButton *m_draw_implicit;
-    wxButton *m_draw_parametric;
-    wxButton *m_draw_points;
-    wxButton *m_draw_fgcolor;
-    wxButton *m_draw_fillcolor;
-    wxButton *m_draw_title;
-    wxButton *m_draw_key;
-    wxButton *m_draw_grid;
-    wxButton *m_draw_axis;
-    wxButton *m_draw_contour;
-    wxButton *m_draw_accuracy;
-    int m_dimensions;
-  };
 public:
   void StatusText(const wxString &text, bool saveInLog = true);
 protected:
@@ -310,53 +276,19 @@ protected:
   //! The default size for the window.
   virtual wxSize DoGetBestClientSize() const;
   //! The sidebar with the draw commands
-  DrawPane *m_drawPane;
+  DrawSidebar *m_drawPane;
 #ifdef USE_WEBVIEW
   HelpBrowser *m_helpPane;
 #endif
-private:
-  class GreekPane : public wxScrolled<wxPanel>
-  {
-  public:
-    GreekPane(wxWindow *parent, Configuration *configuration, Worksheet *worksheet, int ID = wxID_ANY);
-  protected:
-    void UpdateSymbols();
-    void OnMouseRightDown(wxMouseEvent &event);
-    void OnMenu(wxCommandEvent &event);
-    void OnSize(wxSizeEvent &event);
-  private:
-    Configuration *m_configuration;
-    wxSizer *m_lowercaseSizer;
-    wxSizer *m_uppercaseSizer;
-    Worksheet *m_worksheet;
-  };
-
-  class SymbolsPane : public wxScrolled<wxPanel>
-  {
-  public:
-    SymbolsPane(wxWindow *parent, Configuration *configuration, Worksheet *worksheet, int ID = wxID_ANY);
-    //! Update the "user symbols" portion of the symbols pane.
-    void UpdateUserSymbols();
-  protected:
-    void OnMouseRightDown(wxMouseEvent &event);
-    void OnMenu(wxCommandEvent &event);
-    void OnSize(wxSizeEvent &event);
-  private:
-    //! A panel that shows all user-defined symbols on the symbols pane.
-    wxPanel *m_userSymbols;
-    //! A button per user defined symbol
-    std::list<wxWindow *> m_userSymbolButtons;
-    wxSizer *m_userSymbolsSizer;
-    Configuration *m_configuration;
-    Worksheet *m_worksheet;
-  };
-
-  wxPanel *CreateSymbolsPane();
 
+  //! Looks up which demo file belongs to a wxWindowID
+  wxString GetDemoFile(wxWindowID id) const;
 protected:
   std::array<wxString,10> m_statusTextHistory;
   void OnMenuStatusText(wxMenuEvent &event);
-  SymbolsPane *m_symbolsPane;
+  std::unordered_map<wxWindowID, wxString> m_demoFilesIDs;
+
+  SymbolsSidebar *m_symbolsSidebar;
   //! The current length of the evaluation queue of commands we still need to send to maxima
   int m_EvaluationQueueLength;
   //! Do we need to update the display showing the evaluation queue length?
